@@ -1,6 +1,7 @@
 package Net::Dropbox::API;
 
 use common::sense;
+use File::Basename qw(basename);
 use JSON;
 use Mouse;
 use Net::OAuth;
@@ -10,32 +11,32 @@ use Data::Random qw(rand_chars);
 
 =head1 NAME
 
-Net::Dropbox - The great new Net::Dropbox!
+Net::Dropbox::API - A dropbox API interface
 
 =head1 VERSION
 
-Version 0.02
+Version 0.10.10.9.8
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.10';
 
 
 =head1 SYNOPSIS
 
-Quick summary of what the module does.
+The Dropbox API is a OAuth based API. I try to abstract as much away as
+possible so you should not need to know too much about it.
+This is how it works:
 
-Perhaps a little code snippet.
+    use Net::Dropbox::API;
 
-    use Net::Dropbox;
+    my $box = Net::Dropbox::API->new({key => 'KEY', secret => 'SECRET'});
+    my $login_link = $box->login;  # user needs to klick this link and login
+    $box->auth;                    # oauth keys get exchanged
+    my $info = $box->account_info; # and here we have our account info
 
-    my $foo = Net::Dropbox->new();
-    ...
-
-=head1 EXPORT
-
-A list of functions that can be exported.  You can delete this section
-if you don't export anything, such as for a purely object-oriented module.
+See the examples for a working Mojolicious web client using the Dropbox
+API.
 
 =head1 FUNCTIONS
 
@@ -45,21 +46,25 @@ has 'debug' => (is => 'rw', isa => 'Bool', default => 0);
 has 'error' => (is => 'rw', isa => 'Str', predicate => 'has_error');
 has 'key' => (is => 'rw', isa => 'Str');
 has 'secret' => (is => 'rw', isa => 'Str');
-has 'nonce' => (is => 'ro', isa => 'Str', default => join( '', rand_chars( size => 16, set => 'alphanumeric' ) ));
 has 'login_link' => (is => 'rw', isa => 'Str');
 has 'callback_url' => (is => 'rw', isa => 'Str', default => 'http://localhost:3000/callback');
 has 'request_token' => (is => 'rw', isa => 'Str');
 has 'request_secret' => (is => 'rw', isa => 'Str');
 has 'access_token' => (is => 'rw', isa => 'Str');
 has 'access_secret' => (is => 'rw', isa => 'Str');
+has 'context' => (is => 'rw', isa => 'Str', default => 'sandbox');
+
 
 =head2 login
+
 This sets up the initial OAuth handshake and returns the login URL. This
 URL has to be clicked by the user and the the user then has to accept
 the application in dropbox. 
+
 Dropbox then redirects back to the callback URL defined with
-$self->callback_url. If the user already accepted the application the
+C<$self-E<gt>callback_url>. If the user already accepted the application the
 redirect may happen without the user actually clicking anywhere.
+
 =cut
 
 sub login {
@@ -96,9 +101,11 @@ sub login {
 }
 
 =head2 auth
+
 The auth method changes the initial request token into access token that we need
 for subsequent access to the API. This method only has to be called once
 after login.
+
 =cut
 
 sub auth {
@@ -135,7 +142,9 @@ sub auth {
 }
 
 =head2 account_info
+
 account_info polls the users info from dropbox.
+
 =cut
 
 sub account_info {
@@ -144,34 +153,158 @@ sub account_info {
     return from_json($self->_talk('account/info'));
 }
 
-=head2 list_files
-lists all files un the path defined.
-Paths starting with 'dropbox/' refer to the users dropbox and paths
-starting with 'sandbox/' refer to the sandbox directory the user asigned
-to the application.
+=head2 list
+
+lists all files in the path defined.
+
 =cut
-sub list_files {
+
+sub list {
     my $self = shift;
-    my $path = shift;
+    my $path = shift || '';
+
+    return from_json($self->_talk('files/'.$self->context.'/'.$path));
+}
+
+=head2 copy
+
+copies a folder
+    copy($from, $to)
+
+=cut
+
+sub copy {
+    my $self = shift;
+    my ($from, $to) = @_;
+
+    my $opts = 'from_path='.$from.'&to_path='.$to.'&root='.$self->context;
+    return from_json($self->_talk('fileops/copy?'.$opts));
+}
+
+=head2 move
+
+move a folder
+    move($from, $to)
+
+=cut
+
+sub move {
+    my $self = shift;
+    my ($from, $to) = @_;
+
+    my $opts = 'from_path='.$from.'&to_path='.$to.'&root='.$self->context;
+    return from_json($self->_talk('fileops/move?'.$opts));
+}
+
+=head2 mkdir
+
+creates a folder
+    mkdir($path)
+
+=cut
+
+sub mkdir {
+    my $self = shift;
+    my ($path) = @_;
+
+    my $opts = 'path='.$path.'&root='.$self->context;
+    return from_json($self->_talk('fileops/create_folder?'.$opts));
+}
+
+=head2 delete
+
+delete a folder
+    delete($path)
+
+=cut
+
+sub delete {
+    my $self = shift;
+    my ($path) = @_;
+
+    my $opts = 'path='.$path.'&root='.$self->context;
+    return from_json($self->_talk('fileops/delete?'.$opts));
+}
+
+=head2 view
+
+creates a cookie protected link for the user to look at.
+    view($path)
+
+=cut
+
+sub view {
+    my $self = shift;
+    my ($path) = @_;
+
+    return from_json($self->_talk('fileops/links/'.$self->context.'/'.$path));
+}
+
+=head2 metadata
+
+creates a cookie protected link for the user to look at.
+    metadata($path)
+
+=cut
+
+sub metadata {
+    my $self = shift;
+    my $path = shift || '';
+
+    return from_json($self->_talk('metadata/'.$self->context.'/'.$path));
+}
+
+=head2 putfile
+
+uploads a file to dropbox
+
+=cut
+
+sub putfile {
+    my $self     = shift;
+    my $file     = shift;
+    my $path     = shift || '';
+    my $filename = shift || basename( $file );
+
+    return from_json(
+        $self->_talk(
+            'files/'.$self->context.'/'.$path,
+            'POST',
+            { file => [ $file ] },
+            $filename
+        )
+    );
 
-    return from_json($self->_talk('files/'.$path));
 }
 
 
-
 =head1 INTERNAL API
+
 =head2 _talk
+
 _talk handles the access to the restricted resources. You should
 normally not need to access this directly.
+
 =cut
 
+=head2 nonce
+
+Generate a different nonce for every request.
+
+=cut
+
+sub nonce { join( '', rand_chars( size => 16, set => 'alphanumeric' )); }
+
 sub _talk {
-    my $self = shift;
+    my $self    = shift;
     my $command = shift;
-    my $method = shift || 'GET';
+    my $method  = shift || 'GET';
+    my $content = shift;
+    my $filename= shift;
 
     my $ua = LWP::UserAgent->new;
-    my $request = Net::OAuth->request("protected resource")->new(
+
+    my %opts = (
         consumer_key => $self->key,
         consumer_secret => $self->secret,
         request_url => 'http://api.dropbox.com/0/'.$command,
@@ -183,6 +316,12 @@ sub _talk {
         token => $self->access_token,
         token_secret => $self->access_secret,
     );
+    if($filename) {
+        $opts{extra_params}{file} = $filename;
+        $opts{request_url} = 'http://api-content.dropbox.com/0/'.$command;
+    }
+
+    my $request = Net::OAuth->request("protected resource")->new( %opts );
 
     $request->sign;
 
@@ -190,7 +329,7 @@ sub _talk {
     if($method =~ /get/i){
         $res = $ua->get($request->to_url);
     } else {
-        $res = $ua->post($request->to_url);
+        $res = $ua->post($request->to_url, Content_Type => 'form-data', Content => $content );
     }
 
     if ($res->is_success) {
@@ -203,6 +342,7 @@ sub _talk {
     }
     return;
 }
+
 =head2 talk
 
 =cut
@@ -213,19 +353,15 @@ Lenz Gschwendtner, C<< <norbu09 at cpan.org> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-net-dropbox at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Net-Dropbox>.  I will be notified, and then you'll
+Please report any bugs or feature requests to C<bug-net-dropbox-api at rt.cpan.org>, or through
+the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Net-Dropbox-API>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
-
-
-
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
 
-    perldoc Net::Dropbox
-
+    perldoc Net::Dropbox::API
 
 You can also look for information at:
 
@@ -233,26 +369,23 @@ You can also look for information at:
 
 =item * RT: CPAN's request tracker
 
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Net-Dropbox>
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Net-Dropbox-API>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
-L<http://annocpan.org/dist/Net-Dropbox>
+L<http://annocpan.org/dist/Net-Dropbox-API>
 
 =item * CPAN Ratings
 
-L<http://cpanratings.perl.org/d/Net-Dropbox>
+L<http://cpanratings.perl.org/d/Net-Dropbox-API>
 
 =item * Search CPAN
 
-L<http://search.cpan.org/dist/Net-Dropbox/>
+L<http://search.cpan.org/dist/Net-Dropbox-API/>
 
 =back
 
 
-=head1 ACKNOWLEDGEMENTS
-
-
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2010 Lenz Gschwendtner.
